#!/usr/bin/env python

import os
import os.path
import multiprocessing
import time
import random
import xmlrpclib
import cairo
import grp
import sys
import lliurex.net
import traceback


from gi.repository import Gtk, Gdk, GObject, GLib, PangoCairo, Pango

import signal
signal.signal(signal.SIGINT, signal.SIG_DFL)

import gettext
#gettext.textdomain('zero-center')
_ = gettext.gettext

class NetworkConfig:
	
	def __init__(self):
		
		self.client=xmlrpclib.ServerProxy("https://localhost:9779")
		self.process=multiprocessing.Process()
		try:
			if self.client.get_variable("","VariablesManager","INTERFACE_REPLICATION")!=None:
				self.open_dialog(_("Network Configuration"),_("Network reconfiguration is only allowed on independent servers"),"dialog-information")
			else:
				self.start_gui()
		except:
			pass
		
		
	#def init
	
	def start_gui(self):
		
		builder=Gtk.Builder()
		builder.set_translation_domain("llx-network-config")
		if os.path.exists("./rsrc/network-config.glade"):
			builder.add_from_file("./rsrc/network-config.glade")
		else:
			builder.add_from_file("/usr/share/llx-network-config/rsrc/network-config.glade")
		self.window=builder.get_object("window1")
		
		
		
		self.window.connect("destroy",self.window_close)
		self.window.show()
		
		self.internal_speed_label=builder.get_object("internal_iface_speed_label")
		self.external_speed_label=builder.get_object("external_iface_speed_label")

		self.internal_combobox=builder.get_object("internal_combobox")
		self.internal_combobox.set_name("Button")
		self.external_combobox=builder.get_object("external_combobox")
		
		self.internal_ip_entry=builder.get_object("srvip_entry")
		self.internal_mask_entry=builder.get_object("internal_mask_entry")
		
		self.external_ip_entry=builder.get_object("external_ip_entry")
		self.external_mask_entry=builder.get_object("external_mask_entry")
		self.external_gateway_entry=builder.get_object("external_gateway_entry")
		
		
		self.dns1_entry=builder.get_object("dns1_entry")
		self.dns2_entry=builder.get_object("dns2_entry")
		self.dhcp_radiobutton=builder.get_object("dhcp_radiobutton")
		self.dhcp_radiobutton.connect("toggled",self.radio_button_changed)
		self.manual_radiobutton=builder.get_object("manual_radiobutton")
		self.manual_expander=builder.get_object("manual_expander")
		self.manual_expander.set_sensitive(False)
		
		self.apply_button=builder.get_object("apply_button")
		self.apply_button.connect("clicked",self.apply_clicked)
		self.apply_button.set_name("Button")
		
		self.progress_window=builder.get_object("window2")
		self.progress_bar=builder.get_object("progressbar")
		
		ret=self.set_default_gui_values()
		if ret[0]:
			Gtk.main()
		else:
			self.open_dialog(_("Network Configuration"),_("Error getting values.")+"\n[<b>"+str(ret[1])+"</b>]")
		
	#def start_gui
	
	def set_default_gui_values(self):


		try:
			var=self.client.get_variables("","VariablesManager")
			internal=var["INTERNAL_INTERFACE"]["value"]
			external=var["EXTERNAL_INTERFACE"]["value"]
			dns1=var["DNS_EXTERNAL"]["value"][0]
			dns2=var["DNS_EXTERNAL"]["value"][1]
			
		
			self.iiface_model=Gtk.ListStore(str)
			self.eiface_model=Gtk.ListStore(str)
			
			self.internal_combobox.set_model(self.iiface_model)
			self.external_combobox.set_model(self.eiface_model)
			rendi=Gtk.CellRendererText()
			self.internal_combobox.pack_start(rendi,True)
			self.internal_combobox.add_attribute(rendi,"text",0)
			self.internal_combobox.connect("changed",self.get_link_speed,0)
			rende=Gtk.CellRendererText()
			self.external_combobox.pack_start(rende,True)
			self.external_combobox.add_attribute(rende,"text",0)
			self.external_combobox.connect("changed",self.get_link_speed,1)
			self.interfaces=lliurex.net.get_devices_info()		
			
			count=0
			i_id=0
			e_id=0
			for item in self.interfaces:
				if "eth" in item["name"]:
					self.iiface_model.append([item["name"]])
					if item["name"]==internal:
						i_id=count
					self.eiface_model.append([item["name"]])
					if item["name"]==external:
						e_id=count
				count+=1
				
				
					
			self.internal_combobox.set_active(i_id)
			if len(self.iiface_model)>1:
				self.external_combobox.set_active(e_id)
			else:
				self.external_combobox.set_active(0)
				

			if self.is_static(external):
				self.manual_radiobutton.set_active(True)
				


			ip=lliurex.net.get_ip(internal)
			mask=lliurex.net.get_netmask(internal)
			
			self.internal_ip_entry.set_text(ip)
			self.internal_mask_entry.set_text(mask)

				
				
			ip=lliurex.net.get_ip(external)
			self.external_mask_entry.set_text(lliurex.net.get_netmask(external))
			self.external_gateway_entry.set_text(lliurex.net.get_default_gateway()[1])
			ip=ip.split(".")
			self.external_ip_entry.set_text(".".join(ip))
			
			self.dns1_entry.set_text(dns1)
			self.dns2_entry.set_text(dns2)
			
			

				
			return [True,""]
				
		except Exception as e:

			top = traceback.extract_stack()[-1]
			f=top[0]
			line=str(top[1])
			txt="%s at line %s : %s"%(f,line,str(e))
			
			return [False,txt]
			
		
		
	#def set_default_gui_values
	
	def get_link_speed(self,widget,id):
		
		tree_iter = widget.get_active_iter()
		if tree_iter != None:
			model = widget.get_model()
			try:
				speed=lliurex.net.get_device_info(model[tree_iter][0])["Speed"][0]
			except:
				speed="Unknown speed"
			if id==0:
				self.internal_speed_label.set_text(speed)
			else:
				self.external_speed_label.set_text(speed)
				
		
	#def get_link_speed
	
	def is_static(self,eth):
		
		ret=False
		
		try:
		
			f=open("/etc/network/interfaces")
			lines=f.readlines()
			f.close()
			
			for line in lines:
				if eth in line:
					if "static" in line:
						ret=True
			
		except:
			
			pass
			
		return ret
		
	#def is_static

	
	def get_gui_values(self):
		
		var={}
		try:
			tmp=self.client.get_variables("","VariablesManager")
			var["srv_domain_name"]=tmp["INTERNAL_DOMAIN"]["value"]
			var["srv_name"]=tmp["HOSTNAME"]["value"]
			
			var["masterkey"]=self.get_n4d_key()
			var["remote_ip"]="localhost"
			
			iter=self.internal_combobox.get_active_iter()
			if iter!=None:
				var["internal_iface"]=self.iiface_model.get(iter,0)[0]
			else:
				var["internal_iface"]=None
			iter=self.external_combobox.get_active_iter()
			if iter!=None:
				var["external_iface"]=self.eiface_model.get(iter,0)[0]
			else:
				var["external_iface"]=None

			var["srv_ip"]=self.internal_ip_entry.get_text()
			var["internal_mask"]=self.internal_mask_entry.get_text()
			var["external_ip"]=self.external_ip_entry.get_text()
			var["external_mask"]=self.external_mask_entry.get_text()
			var["external_gateway"]=self.external_gateway_entry.get_text()
			var["dns1"]=self.dns1_entry.get_text()
			var["dns2"]=self.dns2_entry.get_text()
					
			
			if self.dhcp_radiobutton.get_active():
				var["external_mode"]="dhcp"
			else:
				var["external_mode"]="manual"
		
			return var
		
		except Exception as e:
			top = traceback.extract_stack()[-1]
			f=top[0]
			line=str(top[1])
			txt="%s at line %s : %s"%(f,line,str(e))
			return txt
		
	#def get_gui_values
	
	def test_values(self,var):
		
		if var["masterkey"]==None:
			return [False,_("You don't have root privileges to run this program")]
		
		if var["internal_iface"]==var["external_iface"]:
			return [False,_("Internal and external interfaces must be different")]
			
		lst=[]
		lst.append((_("Internal IP"),var["srv_ip"]))
		lst.append((_("Internal mask"),var["internal_mask"]))
		lst.append((_("External IP"),var["external_ip"]))
		lst.append((_("External mask"),var["external_mask"]))
		lst.append((_("Gateway"),var["external_gateway"]))
		lst.append((_("DNS #1"),var["dns1"]))
		lst.append((_("DNS #2"),var["dns1"]))
			
		for val in ( lst ):
			txt,item=val
			if not lliurex.net.is_valid_ip(item):
				return [False,_("'<b>%s</b>' must be a valid ip"%txt)]
				
		return [True,None]
		
	#def test_values

	
	def get_n4d_key(self):
		
		try:
			f=open("/etc/n4d/key","r")
			key=f.readline().strip("\n")
			f.close()
			return key
		except:
			None
			
	#def get_n4d_key


	def execute(self):

		msg="* Executing 015-network ... "
		sys.stdout.write(msg)
		execfile("/usr/share/zero-server-wizard/types/independent/actions/015-network.py",locals())
		print("OK")
		msg="* Executing slapd open ports configuration ... "
		sys.stdout.write(msg)
		self.client.open_ports_slapd(self.template["masterkey"],"SlapdManager",self.template["srv_ip"])
		print("OK")
		msg="* Executing 050-dnsmasq ... "
		sys.stdout.write(msg)
		execfile("/usr/share/zero-server-wizard/types/independent/actions/050-dnsmasq.py",locals())
		print("OK")
		msg="* Executing 060-proxy ... "
		sys.stdout.write(msg)
		execfile("/usr/share/zero-server-wizard/types/independent/actions/060-proxy.py",locals())
		print("OK")
		msg="* Restarting services ... "
		sys.stdout.write(msg)
		os.system("/usr/sbin/upstart-manager disable network-manager")
		os.system("stop network-manager")
		os.system("/etc/init.d/networking restart")
		os.system("/etc/init.d/dnsmasq restart")
		os.system("sudo service squid3 restart")
		os.system("sudo service n4d restart")
		print("OK")
		print("")
		print(" ** RECONFIGURATION FINISHED ** ")

	#def execute


	def apply_clicked(self,widget):
		
		var=self.get_gui_values()
		if type(var)!=type(dict()):
			self.open_dialog(_("Network Configuration"),_("Error getting values.")+"\n[<b>"+str(var)+"</b>]")
		ret=self.test_values(var)
		
		if ret[0]:
			
			self.template=var
			self.process=multiprocessing.Process(target=self.execute)
			#t.daemon=True
			self.process.start()
			GLib.timeout_add(100,self.show_progress)
			
		else:

			self.open_dialog(_("Network Configuration"),ret[1])
		
		
	#def apply_clicked
	
	def show_progress(self):
		
		
		self.progress_window.show()
		self.progress_bar.pulse()
		
		if not self.process.is_alive():
			
			self.progress_window.hide()
			self.open_dialog(_("Network Configuration"),_("Configuration finished."),"dialog-information")
			
		return self.process.is_alive()
		
	#def show_progress

	def radio_button_changed(self,widget,force=False):
		
		if force:
			status=force
			
		else:
			status=not self.dhcp_radiobutton.get_active()	
		
		
		self.manual_expander.set_sensitive(status)
		self.manual_expander.set_expanded(status)
		
	#def radio_button_changed
	
	def window_close(self,widget):
		
		Gtk.main_quit()
		
	#def window_close
	
	def open_dialog(self,title,text,icon="emblem-important",show_cancel=False):

		label = Gtk.Label()
		label.set_markup(text)
		if show_cancel:
			dialog = Gtk.Dialog(title, None, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT, (Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT,Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL))
		else:
			dialog = Gtk.Dialog(title, None, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT, (Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))
		hbox = Gtk.HBox()
		img=Gtk.Image.new_from_icon_name(icon,Gtk.IconSize.DIALOG)
		hbox.pack_start(img,True,True,5)
		hbox.pack_start(label,True,True,10)
		hbox.show_all()
		dialog.vbox.pack_start(hbox,True,True,10)
		dialog.set_border_width(6)
		response = dialog.run()
		dialog.destroy()
		return response
		
	#def open_dialog
	

	
#class NetworkConfig



if __name__=="__main__":
	
	nc=NetworkConfig()